/*#######################################################################################
ARM Small Webserver 

Copyright (C) 2006 Ulrich Radig

#######################################################################################*/

#include "stack.h"

unsigned char mymac[6];
unsigned char myip[4];

//############################################################################
//Diese Routine Antwortet auf ein ARP Packet
void ARP_Reply (U08 *buffer,U16 *bufferlen)
//############################################################################
{
	struct Ethernet_Header *ethernet;
	ethernet = (struct Ethernet_Header *)&buffer[ETHER_OFFSET];

	struct ARP_Header *arp;
	arp = (struct ARP_Header *)&buffer[ARP_OFFSET];

	//2 Byte Hardware Typ: Enthlt den Code fr Ethernet
	if(arp->ARP_HWType == 0x0100 &&

	//2 Byte Protokoll Typ: Enthlt den Code fr IP
	arp->ARP_PRType == 0x0008  &&

	//1Byte Lnge der Hardwareadresse:Enthlt 6 fr 6 Byte MAC Addresse
	arp->ARP_HWLen == 0x06 && 

	//1Byte Lnge der Protokolladresse:Enthlt 4 fr 4 Byte Adressen 
	arp->ARP_PRLen == 0x04 &&

	//Operation handelt es sich um eine anfrage
	arp->ARP_Op == 0x0100  &&

	//Ist das ARP Packet fr meine IP Addresse bestimmt
	//Vergleiche ARP Target IP Adresse mit meiner IP
	arp->ARP_TIPAddr == *((U32*)&myip[0]))
		{
		for(U08 a = 0; a < 6; a++)
			{
			//MAC Destadresse wird geschrieben mit MAC Sourceadresse
			ethernet->EnetPacketDest[a] = ethernet->EnetPacketSrc[a];
			//Meine MAC Adresse wird in Sourceadresse geschrieben
			ethernet->EnetPacketSrc[a] = mymac[a];
			//das gleiche fr die MAC Adresse
			//ARP MAC Targetadresse wird geschrieben mit ARP Sourceadresse
			arp->ARP_THAddr[a] = ethernet->EnetPacketDest[a];
			//ARP MAC Sourceadresse wird geschrieben mit My MAC Adresse
			arp->ARP_SHAddr[a] = mymac[a];
			}

		//ARP operation wird auf 2 gesetzt damit der andere merkt es ist ein ECHO	
		arp->ARP_Op = 0x0200;	
		//ARP Target IP Adresse wird geschrieben mit ARP Source IP Adresse 
		arp->ARP_TIPAddr = arp->ARP_SIPAddr;
		//Meine IP Adresse wird in ARP Source IP Adresse geschrieben
		arp->ARP_SIPAddr = *((U32 *)&myip[0]);

		//Nun ist das ARP buffer fertig zum Senden !!!
		//Sendet das erzeugte ARP Packet 
        enc28j60PacketSend(*bufferlen, buffer);
	}
	return;
}

//############################################################################
//Diese Routine erzeugt ein ICMP Packet
void ICMP_Reply (U08 *buffer,U16 *bufferlen)
//############################################################################
{
	//Variablen zur Berechnung der Checksumme
	U16 result16;

	struct IP_Header *ip;
	ip = (struct IP_Header *)&buffer[IP_OFFSET];

	struct ICMP_Header *icmp;
	icmp = (struct ICMP_Header *)&buffer[ICMP_OFFSET];
	
	//Das ist ein Echo Reply Packet
	icmp->ICMP_Type = 0x00;
	icmp->ICMP_Code = 0x00;
	
	//Berechnung der ICMP Checksumme
	//Alle Daten im ICMP Header werden addiert checksum wird deshalb
	//ersteinmal auf null gesetzt
	icmp->ICMP_Cksum = 0x0000;

	//Hier wird erstmal der IP Header neu erstellt
	Make_IP_Header (buffer);

	//Berechnung der ICMP Header lnge
	result16 = ((ip->IP_Pktlen & 0xFF00) >> 8)|((ip->IP_Pktlen & 0x00FF)<<8);
	result16 = result16 - ((ip->IP_Vers_Len & 0x0F) << 2);

	//pointer wird auf das erste Packet im ICMP Header gesetzt
	//jetzt wird die Checksumme berechnet
	result16 = checksum (&icmp->ICMP_Type, result16, 0);
	
	//schreibt Checksumme ins Packet
	icmp->ICMP_Cksum = ((result16 & 0xFF00) >> 8)|((result16 & 0x00FF)<<8);
	
	//Sendet das erzeugte ICMP Packet 
    enc28j60PacketSend(*bufferlen, buffer);
}

//############################################################################
//Diese Routine erzeugt eine Cecksumme
U16 checksum (U08 * pointer,U16 result16,U32 result32)
//############################################################################
{
	U16 result16_1 = 0x0000;
	U08 DataH;
	U08 DataL;
	
	//Jetzt werden alle Packete in einer While Schleife addiert
	while(result16 > 1)
		{
		//schreibt Inhalt Pointer nach DATAH danach inc Pointer
		DataH=*pointer++;

		//schreibt Inhalt Pointer nach DATAL danach inc Pointer
		DataL=*pointer++;

		//erzeugt Int aus Data L und Data H
		result16_1 = ((DataH << 8)+DataL);
		//Addiert packet mit vorherigen
		result32 = result32 + result16_1;
		//decrimiert Lnge von TCP Headerschleife um 2
		result16 -=2;
		}

	//Ist der Wert result16 ungerade ist DataL = 0
	if(result16 > 0)
		{
		//schreibt Inhalt Pointer nach DATAH danach inc Pointer
		DataH=*pointer;
		//erzeugt Int aus Data L ist 0 (ist nicht in der Berechnung) und Data H
		result16_1 = (DataH << 8);
		//Addiert packet mit vorherigen
		result32 = result32 + result16_1;
		}
	
	//Komplementbildung (addiert Long INT_H Byte mit Long INT L Byte)
	result32 = ((result32 & 0x0000FFFF)+ ((result32 & 0xFFFF0000) >> 16));
	result32 = ((result32 & 0x0000FFFF)+ ((result32 & 0xFFFF0000) >> 16));	
	result16 =~(result32 & 0x0000FFFF);
	
return (result16);
}

//############################################################################
//Diese Routine erzeugt ein IP Packet
void Make_IP_Header (U08 *buffer)
//############################################################################
{
	//Variablen zur Berechnung der Checksumme
	U16 result16;

	struct Ethernet_Header *ethernet;
	ethernet = (struct Ethernet_Header *)&buffer[ETHER_OFFSET];

	for (U08 a = 0; a < 6; a++)
		{
		//MAC Destadresse wird geschrieben mit MAC Sourceadresse
		ethernet->EnetPacketDest[a] = ethernet->EnetPacketSrc[a];
		//Meine MAC Adresse wird in Sourceadresse geschrieben
		ethernet->EnetPacketSrc[a] = mymac[a];
		}

	struct IP_Header *ip;
	ip = (struct IP_Header *)&buffer[IP_OFFSET];
	
	//IP Destadresse wird geschrieben mit IP Sourceadresse 
	//das packet soll ja zurckgeschickt werden :-)
	ip->IP_Destaddr	= ip->IP_Srcaddr;
	ip->IP_Srcaddr	= *((U32 *)&myip[0]);
		
	//Berechnung der IP Checksumme
	//Alle Daten im IP Header werden addiert checksum wird deshalb
	//ersteinmal auf null gesetzt
	ip->IP_Hdr_Cksum = 0x0000;
	
	//Berechnung der IP Header lnge	
	result16 = (ip->IP_Vers_Len & 0x0F) << 2;

	//jetzt wird die Checksumme berechnet
	result16 = checksum (&ip->IP_Vers_Len, result16, 0);

	//schreibt Checksumme ins Packet
	ip->IP_Hdr_Cksum = ((result16 & 0xFF00) >> 8)|((result16 & 0x00FF)<<8);
}
